/*  functions.h

This library has some simple functions we want to use.

*/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include <stdlib.h>

#define SQR(x)     ((x)*(x))
#define CUBE(x)    ((x)*(x)*(x))
#ifndef PI
#define PI         3.14159
#endif

void CROSS(float v[3],float w[3], float ans[3])
{
  ans[0]=-v[2]*w[1]+v[1]*w[2];
  ans[1]=-v[0]*w[2]+v[2]*w[0];
  ans[2]=-v[1]*w[0]+v[0]*w[1];
//  fprintf (stderr,"cross %f %f %f  \n",ans[0],ans[1],ans[2]);
//  return ans;
}

void VECT(float a, float b, float c, float ans[3])
{
  ans[0]=a;
  ans[1]=b;
  ans[2]=c;
//  fprintf (stderr,"vect %f %f %f  ",ans[0],ans[1],ans[2]);
}

void DIFF(float v[3],float w[3], float ans[3])
{
  ans[0]=v[0]-w[0];
  ans[1]=v[1]-w[1];
  ans[2]=v[2]-w[2];
//  fprintf (stderr,"diff %f %f %f  ",ans[0],ans[1],ans[2]);
//  return ans;
}

void NORMAL(float a[3],float b[3],float c[3], float d[3])
{
  float dir1[3],dir2[3];
  DIFF(b,a,dir1);
  DIFF(c,a,dir2);
  CROSS(dir1,dir2,d);
  float length=sqrt(SQR(d[0])+SQR(d[1])+SQR(d[2]));
  d[0]/=length;
  d[1]/=length;
  d[2]/=length;
//  fprintf (stderr,"cross %f %f %f  \n",d[0],d[1],d[2]);
}

void ROT_AXIS(float axis[3],float ang,float orig[3],float rot[3])
{
  /*  rot[0]=orig[0]*cos(dy)*cos(dz) +orig[1]*sin(dz) -orig[2]*sin(dz)*sin(dy);
  rot[1]=-orig[0]*sin(dx)*sin(dz) +orig[1]*cos(dx)*cos(dz) +orig[2]*sin(dx);
  rot[2]=orig[0]*sin(dy) -orig[1]*sin(dy)*sin(dx) +orig[2]*cos(dx)*cos(dy);
  
  rot[1]=orig[1]*cos(dx)+orig[2]*sin(dx);
  rot[2]=-orig[1]*sin(dx)+orig[2]*cos(dx);

  rot[0]=orig[0]*cos(dy)-orig[2]*sin(dy);
  rot[2]=orig[0]*sin(dy)+orig[2]*cos(dy);
  rot[0]=orig[0]*cos(dz)+orig[1]*sin(dz);
  rot[1]=-orig[0]*sin(dz)+orig[1]*cos(dz);
  */

  float c=cos(ang);
  float s=sin(ang);
  float x,y,z,v1,v2,v3;
  x=axis[0];
  y=axis[1];
  z=axis[2];
  v1=orig[0];
  v2=orig[1];
  v3=orig[2];

  rot[0]=(x*x*(1-c)+c)*v1 + (x*y*(1-c)-z*s)*v2 + (x*z*(1-c)+y*s)*v3;
  rot[1]=(y*x*(1-c)+z*s)*v1 + (y*y*(1-c)+c)*v2 + (y*z*(1-c)-x*s)*v3;
  rot[2]=(x*z*(1-c)-y*s)*v1 + (y*z*(1-c)+x*s)*v2 + (z*z*(1-c)+c)*v3;
}

void ROTATE(float dx,float dy,float dz,float orig[3],float rot[3])
{
  float x[3]={1,0,0};
  float y[3]={0,1,0};
  float z[3]={0,0,1};
  float tmp[3];
  
  ROT_AXIS(x,dx,orig,tmp);
  ROT_AXIS(x,dx,y,y);
  ROT_AXIS(x,dx,z,z);
  ROT_AXIS(y,dy,tmp,tmp);
  ROT_AXIS(y,dy,x,x);
  ROT_AXIS(y,dy,z,z);
  ROT_AXIS(z,dz,tmp,rot);

}

void LINEUP(float dx, float dy, float dz, float orig[3], float rot[3])
{
  float x[3]={1,0,0};
  float y[3]={0,1,0};
  float z[3]={0,0,1};
  float tmp[3];

  ROT_AXIS(y,dy,orig,tmp);
  ROT_AXIS(y,dy,x,x);
  ROT_AXIS(x,dx,tmp,tmp);
  ROT_AXIS(y,dy,z,z);
  ROT_AXIS(x,dx,z,z);
  ROT_AXIS(z,dz,tmp,rot);
}

void CIRCLE_PTS (int phi, int theta, float points[][3])
{
  float temp[3]={0,1,0};
  float tempold[3]={0,-1,0};
  int i,j,k;

  points[0][0]=temp[0];
  points[0][1]=temp[1];
  points[0][2]=temp[2];
  points[1+(phi-2)*theta][0]=tempold[0];
  points[1+(phi-2)*theta][1]=tempold[1];
  points[1+(phi-2)*theta][2]=tempold[2];
  //  fprintf(stderr,"got the first points\n");

  k=1;
  
  for (i=0;i<phi-2;i++)
    {
    tempold[0]=0;
    tempold[1]=1;
    tempold[2]=0;
    ROTATE((float)((i+1)*PI)/(float)(phi-1),0,0,tempold,temp);
    //    fprintf(stderr,"phi rotation %d\n",i+1);
    for (j=0;j<theta;j++)
      {
      tempold[0]=temp[0];
      tempold[1]=temp[1];
      tempold[2]=temp[2];
      ROTATE(0,(float)((j)*2*PI)/(float)theta,0,tempold,temp);
      //      fprintf(stderr,"theta rotation %d\n",j+1);
      points[k][0]=temp[0];
      points[k][1]=temp[1];
      points[k++][2]=temp[2];
      //     fprintf(stderr,"point %d done\n",k);
      }
    }
}

#endif
