/**********************************************

  COSMIC DUST

  (game.c++)

  This CAVE game was developed and written by Pranas Pauliukonis and 
  Jason Bestimt fall 1997-spring 1998.  Here's the deal:  you can use this
  code any way you want, as long as you give Pranas and Jason their credit
  for any code you borrow, and as long as you don't make money off it (unless
  Pranas and Jason get a fair cut of any profits, of course).  We would like
  to thank John Shalf for his assistance in learning to program for the CAVE 
  (as well as his shared.h code), as well as the helpful VSS developers (as
  sound makes this game about 300% better... sorry I forgot your names!).

  The object of this game is simple:  shoot things and blow them up.

  Controls:  Left wand button: start level (add ship)
             Middle wand button: shoot
	     Right wand button: thrusters
	     Ship always points in the direction the wand is pointing.

  Anyone who would like to improve this game please contact Pranas at 
  pranas@vpst.org     Some improvements we would like to see include 
  networking, getting the bombs to work, possibly adding more weapons, 
  adding a soundtrack, etc.  Since this code isn't very well commented, 
  Pranas is willing to explain any of the strange things he had to program 
  to get this game to work.

  ***********************************************/

#define SOUND_ON
#define TEXSIZEX 2048
#define TEXSIZEY 4096

#include <cave_ogl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#ifdef SOUND_ON
#include "vssClient.h"
#endif
#include "our_colors.h"      //colors file
#include "shared.h"          //shared memory class
#include "object.h"          //base class
#include "head.h"            //head of a linked list
//#include "sphere.h"          //asteroid class
#include "rock.h"            //new asteroid class
#include "ship.h"            //our ship
#include "bullet.h"          //bullets to be fired class
#include "system.h"          //control class
#include "badguy.h"          //badguy ship class

#define dt .0001             //change in time during game

GLubyte star_tex[TEXSIZEX][TEXSIZEY][4];   //texture for starfield
GLubyte earth_tex[512][1024][4];                  //texture for planet

System *Game;                //we need a system to run our game
void exit_application(int);

///////////////////////////////////////////////////

static GLUquadricObj *sphereObj;       //used for open GL quadric
static GLuint texName;
static GLuint ERFtexName;

void init_star_tex()
{
  FILE *fp;
  int i,j;
  short temp[1024][512][4];
  for (i=0;i<TEXSIZEX;i++)
    for (j=0;j<TEXSIZEY;j++)
      if (((float)rand())/16384 <= 0.001)
	{
//	fprintf(stderr,"WHITE POINT     ");
	star_tex[i][j][0]=255;
	star_tex[i][j][1]=255;
	star_tex[i][j][2]=255;
	star_tex[i][j][3]=255;
	}
      else 
	{
	star_tex[i][j][0]=(GLubyte)0;
	star_tex[i][j][1]=(GLubyte)0;
	star_tex[i][j][2]=(GLubyte)0;
	star_tex[i][j][3]=(GLubyte)255;
	}
/*
  for (i=0;i<200;i++)
    for (j=0;j<200;j++)
      fprintf(stderr,"%d %d %d %d ",star_tex[i][j][0], star_tex[i][j][1],star_tex[i][j][2],star_tex[i][j][3]);    
*/

  if (fp=fopen("out.mine","rb"))
    fread((void*)temp,sizeof(short),1024*512*4,fp);

  for (i=0;i<512;i++)
    for (j=0;j<1024;j++)
      {
      earth_tex[i][j][0]=(GLubyte)temp[j][i][0];
      earth_tex[i][j][1]=(GLubyte)temp[j][i][1];
      earth_tex[i][j][2]=(GLubyte)temp[j][i][2];
      earth_tex[i][j][3]=(GLubyte)temp[j][i][3];
      }
  for (i=0;i<512;i++)
    fprintf(stderr,"%d %d %d %d\n",(int)earth_tex[500][i][0],(int)earth_tex[500][i][1],(int)earth_tex[500][i][2],(int)earth_tex[500][i][3]);

  fclose(fp);

}


/////OpenGL Initialization Procedure//////
/* init_gl - initialize GL lighting & materials */
void init_gl(void)
{
  float redMaterial[] = { 1, 0, 0, 1 };     //set up read material
 glEnable(GL_LIGHT0);
 glMaterialfv(GL_FRONT_AND_BACK,
	      GL_AMBIENT_AND_DIFFUSE,
	      redMaterial);
 CAVEFar=100000;
//  gluQuadricTexture(sphereObj,GL_TRUE);
 sphereObj = gluNewQuadric();
  gluQuadricTexture(sphereObj,GL_TRUE);
  glGenTexturesEXT(1,&texName);
  glGenTexturesEXT(1,&ERFtexName);
  glBindTextureEXT(GL_TEXTURE_2D, texName);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TEXSIZEX, TEXSIZEY, 0,GL_RGBA, GL_UNSIGNED_BYTE, star_tex);
  glBindTextureEXT(GL_TEXTURE_2D, ERFtexName);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 1024, 512, 0,GL_RGBA, GL_UNSIGNED_BYTE, earth_tex);
}

/////////////////////////////////////////////////////
////CAVE Draw Process///////
////Calls the System Objects DrawWorld Function
////System is in shared memory and therefore needs no parameters

void Draw()
{
  Game->DrawWorld();          //draw back function
}

////////////////////////////////////////////////////
/////MAIN!!!! This is where it all begins///////////

void main(int argc,char **argv)
{
  SharedMemArena::Init(megs(6));      //our shared memory is 6 megabytes
  SharedMemArena::on();               //new variables will be in shared memory

  init_star_tex();

  Game = new System();                //create a system object to put in pointer
  srand(100);                         //for random numbers
  signal(SIGINT, SIG_IGN);            //used by NCSA for Ctrl '\' for quits
  CAVEConfigure(&argc,argv,NULL);     //set up cave options based on command line
  CAVEInit();                         //sets up multiple processing threads
  signal(SIGINT, exit_application);   //if a Ctrl '\' exit application
  CAVEInitApplication(init_gl,0);     //set Open Gl initialization procedure
  CAVEDisplay((CAVECALLBACK)Draw,0);  //set up draw function and paramters

#ifdef SOUND_ON                       //if sound was declared 
  if (!BeginSoundServer())            //if sound server not started
    {
      printf("No connection to sound server!\n\n");  //then print this message
    }
#endif SOUND_ON                       //end if

  Game->InitWorld();                  //set up game world for play

  while ((!CAVEgetbutton(CAVE_ESCKEY))&&
	 ((CAVEBUTTON1==0)||(CAVEBUTTON2==0)||(CAVEBUTTON3==0))) 
               //while they don't press esc or 3 buttons and quit
    {
      
      Game->Run();                    //run a game step
      
      sginap(1);                      //give the machine time to breath
    }
  Game->DestroyWorld();               //clean up linked lists
  Game->KillSound();                  //delete sound server
  Game->~System();                    //destroy the system object
  SharedMemArena::off();              //turn shared memory off
  CAVEExit();                         //exit our wonderful cave game
}

///another way to quit our wonderful game

void exit_application(int temp)       
{
  CAVEExit();                         //quit
}









