#include <cave_ogl.h>
#include <GL/glu.h>
#include "functions.h"
#include <math.h>

#ifndef OBJECT_H
#define OBJECT_H

float K=10000;

/*************Class Declaration**************/

class Object{
protected:
	float Pos[3];					//x,y,z
	float Vel[3];					//vx,vy,vz
	float Acc[3];					//ax,ay,az
	float RotAng[3];  				//Rotational angle in ??????
	float RotVel[3];				//Rotational velocity ????
	float RotAcc[3];				//Rotational acceleration
	float Orient[3];
	Object *nextobj;
	int   GravOn;                                   //Does gravity affect this object
	float RadiusOfCollision;                        //the radius at which it collides with something
	int CollidedThisTick;
	int CollidedLastTick;
	int Weapon;
	int BadGuy;
	int Locked;
	int Death;
	int RemoveOnExplosion;
public:
	Object();					//Constructor
	~Object();					//Destructor
	virtual void Destroy()=0;			//Derived class destructor
	
	/////LINKED LIST FUNCTIONS							
	void Link(Object *next);			//Links to next object
	Object *Next();					//Moves to next object in list
	
        /////DRAWING FUNCTIONS
	virtual void Draw()=0;				//Derived class drawing function
	virtual void FileWrite()=0;			//Derived class storing function
	void Move(float x, float y, float z);           //Move it
	void MoveRel(float dx, float dy, float dz);     //Move from where you are
	void SetVel(float vx, float vy, float vz);      //You vil moof zis vast
	void AddVel(float dx, float dy, float dz);      //Faster
	void Rot(float rx, float ry, float rz);         //Turn
	void RotRel(float drx, float dry, float drz);   //A little to the left
	void SetOrient(float dx, float dy, float dz);   //Twist
	void AddOrient(float dx, float dy, float dz);   //Twist Again, (like we did last summer)
	
	//////ACCELERATION FUNCTIONS
	void ResetAccel();                              //no acceleration
	void TurnGravOn(){GravOn=1;}                    //feel the pull
	void TurnGravOff(){GravOn=0;}                   //sit, beg, play dead
	int  IsGravOn(){return GravOn;}                 //anyone there
	void CalcAccel(Object *obj);                    //react to another object

	/////MASS OF AN OBJECT
	virtual float GetMass()=0;                      //every object needs to go to church
	
	/////USED IN THE TICK FUNCTION 
	void CalcVel(float dt);                         //da = dv / dt
	void CalcPos(float dt);                         //dx = da / dt

	/////TICK FUNCTION USED FOR INCREMENTING ALL STATS
        virtual void Tick(float dt);                    //do your stuff in your time

	/////COLLISION DETECTION ALGORITHMS
	void  SetRadiusOfCollision(float rad){RadiusOfCollision=rad;}
	float DistanceToObject(Object *other);          //are we there yet
	int   CollisionDetected(Object *other);         //boom
	void  ResolveCollision(Object *other);          //fix this mess
	int   PreviousCollision(){return(CollidedLastTick);};  //has collided already
	int   IsWeapon(){return Weapon;};               //My kung-fu is the best
	int   IsBadGuy(){return BadGuy;};               //Grrrrrr
	virtual Object *Explode ()=0;                   //Explode if you can
	virtual int TrashIt()=0;                        //Destroy if needed
	void  MakeUnExplodable(){RemoveOnExplosion=0;}; //Kung-fu style is undefeatable
	int   RemoveMe(){return(RemoveOnExplosion);};   //Get out of here

	/////DATA FUNCTIONS
	float GetX(){return Pos[0];};                   //where am I?
	float GetY(){return Pos[1];};
	float GetZ(){return Pos[2];};
	float GetVelX(){return Vel[0];};                //where am I going?
	float GetVelY(){return Vel[1];};
	float GetVelZ(){return Vel[2];};

	/////MEMORY ACCESS FUNCTIONS
	int IsLocked(){return(Locked);};                //hands off?
	void Lock(){Locked=1;};                         //look but don't touch
	void UnLock(){Locked=0;};                       //free at last
        void Killing(){Death=1;};
        int IsKilling(){return(Death);};

	virtual int GetBulletData(float locate[],float velocity[]){return 0;};
	virtual int Points(){return 0;};                

};

/****************Contructor*************/

Object::Object()
{
	Pos[0]=0;               //the following is default position stuff
	Pos[1]=0;
	Pos[2]=0;
        Vel[0]=0;
        Vel[1]=0;
        Vel[2]=0;
	Acc[0]=0;
	Acc[1]=0;
	Acc[2]=0;
	RotAng[0]=0;
	RotAng[1]=0;
	RotAng[2]=0;
        RotVel[0]=0;
        RotVel[1]=0;
        RotVel[2]=0;
	RotAcc[0]=0;
	RotAcc[1]=0;
	RotAcc[2]=0;
	Orient[0]=0;
	Orient[1]=0;
	Orient[2]=0;
	RemoveOnExplosion=1;     //get rid of that sucker
	nextobj=NULL;            //no internal link
	GravOn = 1;              //we have gravity
	RadiusOfCollision = 0;   //no collision radius for this object
	CollidedThisTick=0;      //haven't collided yet
        Weapon=0;                //most things aren't weapons
	BadGuy=0;
	Locked=0;
        Death=0;
}

/****************Destructor**************/

Object::~Object(){}     //Do nothing

/****************Linking*****************/

void Object::Link(Object *next)
{
	nextobj=next;    //put new sucker at the butt of this one
}

/*****************************************/

Object* Object::Next()
{
	return(nextobj);  //next please
}

/*****************************************/

void Object::Move(float x, float y, float z)
{
	Pos[0]=x;     //set position to x,y,z
	Pos[1]=y;
	Pos[2]=z;
}

/*****************************************/

void Object::MoveRel(float dx, float dy, float dz)
{
        Pos[0]+=dx;          //add dx,dy,dz to our position 
	Pos[1]+=dy;
	Pos[2]+=dz;
}

/*****************************************/

void Object::SetVel(float vx, float vy, float vz)
{
	Vel[0]=vx;           //new speed
	Vel[1]=vy;
	Vel[2]=vz;
}

/*****************************************/

void Object::AddVel(float dx, float dy, float dz)
{ 
	Vel[0]+=dx;          //add to speed
	Vel[1]+=dy;
	Vel[2]+=dz;
}

/*****************************************/

void Object::Rot(float rx, float ry, float rz)
{
	//Guys, you need to mod by 2pi
	RotAng[0]=rx;
	RotAng[1]=ry;
	RotAng[2]=rz;
}	

/*****************************************/

void Object::RotRel(float drx, float dry, float drz)
{
	//Guys, you need to mod by 2pi
	RotAng[0]+=drx;
	RotAng[1]+=dry;
	RotAng[2]+=drz;
}

/*****************************************/

void Object::SetOrient(float dx, float dy, float dz)
{
  Orient[0]=dx;         //twist
  Orient[1]=dy;
  Orient[2]=dz;
}

/*****************************************/

void Object::AddOrient(float dx, float dy, float dz)
{
  Orient[0]+=dx;        //more twist
  Orient[1]+=dy;
  Orient[2]+=dz;
}

/*****************************************/

void Object::ResetAccel()
{
        Acc[0]=0;          //stop running in the house
	Acc[1]=0;
	Acc[2]=0;
}

/*****************************************/

void Object::CalcAccel(Object *obj)
{
        float G=0.25;
	float mass=obj->GetMass();      //how heavy are you
	float distsqr=SQR(obj->Pos[0] - this->Pos[0]) + SQR(obj->Pos[1] - this->Pos[1]) + SQR(obj->Pos[2] - this->Pos[2]);

	if (distsqr !=0)        //if the object is not me
	  {
 	   this->Acc[0]+=G*((obj->Pos[0] - this->Pos[0])/sqrt(distsqr))*mass/distsqr;
	   this->Acc[1]+=G*((obj->Pos[1] - this->Pos[1])/sqrt(distsqr))*mass/distsqr;
	   this->Acc[2]+=G*((obj->Pos[2] - this->Pos[2])/sqrt(distsqr))*mass/distsqr;
	  }
}

/*****************************************/

void Object::CalcVel(float dt)
{
	Vel[0]+=Acc[0]*dt;           //use dt to accelerate
	Vel[1]+=Acc[1]*dt;
	Vel[2]+=Acc[2]*dt;
	RotVel[0]+=RotAcc[0]*dt;
	RotVel[1]+=RotAcc[1]*dt;
	RotVel[2]+=RotAcc[2]*dt;
}

/*****************************************/

void Object::CalcPos(float dt)
{
	Pos[0]+=Vel[0]*dt;           //use velocity to find change in position
	Pos[1]+=Vel[1]*dt;
	Pos[2]+=Vel[2]*dt;
	RotAng[0]+=RotVel[0]*dt;
	RotAng[1]+=RotVel[1]*dt;
	RotAng[2]+=RotVel[2]*dt;
}

/*****************************************/

void Object::Tick(float dt)
{
  this->CalcVel(dt);                        //find velocity
  this->CalcPos(dt);                        //find pos
  if(CollidedThisTick)                      //I collided and my cat's breath smells like cat food
    CollidedLastTick=1;
  else CollidedLastTick=0;
    this->CollidedThisTick=0;               //reset for next tick
}

/*****************************************/

float Object::DistanceToObject(Object *other)
{
  float dx,dy,dz;                        //how far is it now
  dx = this->Pos[0] - other->Pos[0];
  dy = this->Pos[1] - other->Pos[1];
  dz = this->Pos[2] - other->Pos[2];
  return sqrt(dx*dx + dy*dy + dz*dz);    //good ole pythag
}

/*****************************************/

int  Object::CollisionDetected(Object *other)
{
  int collided=0;
  float dist;
  dist = this->DistanceToObject(other);
  if (this!=other)          //same object
      if (dist <= this->RadiusOfCollision + other->RadiusOfCollision)
        collided=1;               //they are too close for comfort
  return collided;
}

/*****************************************/

void Object::ResolveCollision(Object *other)
{
  float m1,m2,p1x,p1y,p1z,p2x,p2y,p2z,norm,dist;
  m1=this->GetMass();
  m2=other->GetMass();
  //our masses
  
  p1x = this->Pos[0];
  p1y = this->Pos[1];
  p1z = this->Pos[2];
  p2x = other->Pos[0];
  p2y = other->Pos[1];
  p2z = other->Pos[2];
  norm=sqrt(SQR(p1x-p2x)+SQR(p1y-p2y)+SQR(p1z-p2z));
  dist=this->RadiusOfCollision + other->RadiusOfCollision - norm;

  //use spring constant to find reaction force
  this->Acc[0]+=((p1x-p2x)/norm)*K*dist/m1;
  this->Acc[1]+=((p1y-p2y)/norm)*K*dist/m1;
  this->Acc[2]+=((p1z-p2z)/norm)*K*dist/m1;
  other->Acc[0]-=((p1x-p2x)/norm)*K*dist/m2;
  other->Acc[1]-=((p1y-p2y)/norm)*K*dist/m2;
  other->Acc[2]-=((p1z-p2z)/norm)*K*dist/m2;

  //we have collision
  this->CollidedThisTick=1;
  other->CollidedThisTick=1;

}

#endif
