#include "object.h"

#ifndef PLANET_H
#define PLANET_H
extern GLUquadricObj *sphereObj;
extern GLubyte earth_tex[512][1024][4];   //texture for starfield
extern GLuint ERFtexName;

#define NUMPHI 16
#define NUMTHETA 16

/*************Class Declaration**************/

class Planet:public Object{
private:
	float radius;
	float color[4];
	float density;
public:
	Planet(float r,float d);
	Planet(float r);
	Planet();
	~Planet();
	void Destroy();
	void Draw();	
	void FileWrite();
	float GetMass();
	void SetColor(float r, float g, float b, float a);
	void SetColor(float c[]);
	void SetVel(float v[]);
	void SetVel(float vx,float vy, float vz);
	Object *Explode(){return NULL;};
	int TrashIt(){return 0;};
};

/****************Contructor*************/

Planet::Planet(float r,float d):Object()
{
	radius=r;
	density=d;
	this->SetColor(.2,.2,.2,1);
}

/****************Contructor*************/

Planet::Planet(float r):Object()
{
	radius=r;
	density=25;
	this->SetColor(.2,.2,.2,1);
}

/*********Default Constructor***********/

Planet::Planet():Object()
{
  radius=1;
  density=25;
  this->SetColor(.2,.2,.2,1);
}

/****************Destructor*************/

Planet::~Planet(){}

/****************************************/

void Planet::Destroy()
{
	this->~Planet();
}

/****************************************/

void Planet::Draw()
{
        glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,white);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,white);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,64);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glBindTextureEXT (GL_TEXTURE_2D, ERFtexName);
  glEnable(GL_TEXTURE_2D);
  glPolygonMode(GL_BACK,GL_LINE);
  gluQuadricTexture(sphereObj,GL_TRUE);
	glPushMatrix();
	glTranslatef(Pos[0],Pos[1],Pos[2]);
	glRotatef(-90,1,0,0);
	if (radius > 1)
	  gluSphere(sphereObj,radius,32,32);   //used to be 8,8
	else if (radius > .5)
          gluSphere(sphereObj,radius,16,16);
	else
	  gluSphere(sphereObj,radius,8,8);
  glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	glPolygonMode(GL_BACK,GL_FILL);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,0);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,black);
}

/****************************************/

void Planet::FileWrite()
{

}

/****************************************/

float Planet::GetMass()
{
  return (density*CUBE(radius)*3/4);
}

/*****************************************/

void Planet::SetColor(float r, float g, float b, float a)
{
  color[0]=r;
  color[1]=g;
  color[2]=b;
  color[3]=a;
}

/******************************************/

void Planet::SetColor(float c[])
{
  color[0]=c[0];
  color[1]=c[1];
  color[2]=c[2];
  color[3]=c[3];
}

/*****************************************/

void Planet::SetVel(float v[])
{
  Vel[0]=v[0];
  Vel[1]=v[1];
  Vel[2]=v[2];
}

/*****************************************/

void Planet::SetVel(float vx, float vy, float vz)
{
  Vel[0]=vx;
  Vel[1]=vy;
  Vel[2]=vz;
}



#endif
