#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ulocks.h>
#include <fcntl.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <time.h>
#include <malloc.h>
#include <bstring.h>

#ifndef SHARED_H
#define SHARED_H

inline int megs(int i) {return (i * 1024 * 1024);}

//////////////////////////////////////////////////
//Shared Memory Allocation class
//Lifted from Code by John Shalf 
//who lifred it from John Goldman of UIC EVL
//who lifted it from Terry Frenguiadakis
//who lifted it from Jon Goldman
//
//Purpose:
//Simplifies shared memory allocation.
//Replaces new and delete operators so that
//the internal allocations in opaque c++ libs
//will allocate out of shared mem as well
///////////////////////////////////////////////

class SharedMemArena {
 protected:
   static int ShmOn;
   static char *name;
   static usptr_t *Arena;
   static void *arena;
   static int did_it;
   static int size, freestore;
 public:
   SharedMemArena( size_t size );
   SharedMemArena( void *(*)(int), size_t );
   ~SharedMemArena();

   static void perror (char *);
   static void Init(size_t size);
   static void load_arena ( void *ar, int sz );

   static char *adres;

   static void *malloc( size_t t);
   static void free(void *p);
   static void *realloc(void *p, size_t t);

   static void *snew(size_t t);
   static void sdelete(void *&p);
   static void *srealloc(void *&p,size_t t);

   //inline safe accessors for private data
   static inline void on(void) {if(did_it) ShmOn=1;}
   static inline void off(void) {ShmOn=0;}
   static inline int isOn() {return ShmOn;}
   static inline int isOff() {return (ShmOn?0:1);}
};

///////////////////////////member function definitions

void    *SharedMemArena::arena = (void *)1;
usptr_t *SharedMemArena::Arena = (void *)1;
char    *SharedMemArena::adres = 0;
char    *SharedMemArena::name  = "/tmp/SharedMemArena";
int      SharedMemArena::did_it= 0;
int      SharedMemArena::size  = 0;
int      SharedMemArena::ShmOn = 0;
long     memalloced = 0;

////////////////////////////////////

SharedMemArena::SharedMemArena( size_t sz)
{
  if(!did_it)
    {
      Init(sz);
      did_it=1;
      size = sz;
    }
  else
    perror (" second instance of SharedMemArena ");
}

////////////////////////////////////////////

SharedMemArena::SharedMemArena(void *(*arenamaker)(int), size_t sz)
{
  size = sz;
  SharedMemArena::arena = arenamaker(size);
}

///////////////////////////////////////////

SharedMemArena::~SharedMemArena()
{
  adelete(SharedMemArena::arena);
}

////////////////////////////////////////////

void SharedMemArena::load_arena(void *ar, int sz)
{
  SharedMemArena::arena = ar;
  SharedMemArena::size = sz;
}

///////////////////////////////////////////////

void SharedMemArena::Init (size_t sz)
{
  char *     shmemFile = "/dev/zero";
  long       shmemFd;
  void       *_shmemArena, *shmemAddr;

  if (did_it)
    {
      did_it++;
      return;
    }
  else
    did_it++;
  if ((shmemFd = open(shmemFile, O_RDWR|O_CREAT)) < 0)
    {
      perror ("open");
      SharedMemArena::arena = NULL;
    }
#ifndef MAP_AUTORESV
#define MAP_AUTORESV 0x100
#endif
  if((int)(shmemAddr = mmap(SharedMemArena::adres, sz,
			    PROT_READ|PROT_WRITE,
			    MAP_SHARED|MAP_AUTOGROW|
			    MAP_AUTORESV, shmemFd, 0)) <0)
    {
      perror("mmap");
      close(shmemFd);
      SharedMemArena::arena = NULL;
    }
  close (shmemFd);
  if (!(Arena = usinit(SharedMemArena::name)))
    {
      SharedMemArena::arena = NULL;
    }
  unlink(SharedMemArena::name);
  if (!(_shmemArena = acreate(shmemAddr,sz,MEM_SHARED,Arena,NULL)))
    {
      perror("acreate");
      SharedMemArena::arena = NULL;
    }
  SharedMemArena::arena = _shmemArena;
}

////////////////////////////////////////////////////

void *SharedMemArena::malloc(size_t t)
{
  if (ShmOn)
    return amalloc(t, arena);
  else
    return malloc(t);
}

/////////////////////////////////////////////////////

void SharedMemArena::free(void *p)
{
  if (ShmOn)
    afree(p,arena);
  else
    free(p);
}

//////////////////////////////////////////////////////

void *SharedMemArena::realloc(void *p,size_t t)
{
  if (ShmOn)
    return arealloc(p,t,arena);
  else
    return realloc(p,t);
}

////////////////////////////////////////////////////////

void *SharedMemArena::snew(size_t t)
{
  if (did_it)
    return amalloc(t,arena);
  else
    return malloc(t);
}

/////////////////////////////////////////////////////////

void SharedMemArena::sdelete(void *&p)

{
  if (did_it)
    {
      afree(p,arena);
      p=NULL;
    }
  else
    {
      free(p);
      p=0;
    }
}
  
//////////////////////////////////////////////////////////

void *SharedMemArena::srealloc(void *&p,size_t t)
{
  if (did_it)
    return p=arealloc(p,t,arena);
  else
    return p=realloc(p,t);
}

void SharedMemArena::perror(char *errormsg)
{
  cerr << errormsg <<endl;
}

///////////////////////////////////////////////////////
///////////////////////////////////////////////////////

//Redefine new and delete
void *operator new( size_t size)
{
  if (SharedMemArena::isOn())
    return SharedMemArena::malloc(size);
  else
    return malloc(size);
}

///////////////////////////////////

void *snew( size_t size)
{
  return SharedMemArena::snew(size);
}

///////////////////////////////////

void operator delete(void *p)
{
  if (SharedMemArena::isOn())
    SharedMemArena::free(p);
  else
    free(p);
}

///////////////////////////////////

void sdelete(void *&p)
{
  SharedMemArena::sdelete(p);
}

///////////////////////////////////

void *srealloc(void *&p, size_t size)
{
  return SharedMemArena::srealloc(p,size);
}

#endif ///SHARED_H
