#include "object.h"
#include "our_colors.h"

#ifndef SHIP_H
#define SHIP_H

/*************Class Declaration**************/

class Ship:public Object{
private:
	float color[4];           //What color ship do YOU want?  Mauve?  Chartreuse?
				  //How about this *stunning* gray color???
	float thrust;             //Oh, my...
	float GunPos[3];          //Where the bullets come out of
	float RadarPos[3];        //Where the radar is sitting... see ERGONOMICS
	float scale;              //We didn't make this ship THAT BIG, did we?
public:
	Ship();                   //You should recognize these...
	~Ship();
	void Destroy();
	void Draw();	
	void FileWrite();
	float GetMass();          //Man, this ship is a hog..
	void SetColor(float r, float g, float b, float a);     //Hmm.. psychadelic ship... that's for v2.0...
	void SetColor(float c[]);                              //The ever popular vector form of the above function
	void CheckThrust();                                  //You want to GO somewhere?
	int GetBulletData(float locate[],float velocity[]);  //The bullet constructor needs this stuff
	void GetRadarPos(float rpos[]);                      //Gives the location of the radar to System->DrawWorld()
	Object *Explode ();                                  //AAAAAAAAAAAAAAAAHHHH!!!!
	int TrashIt(){return 0;};                          //we ain't goin nowhere
};

/****************Default Contructor*************/

Ship::Ship():Object()
{
  SetColor(1,1,1,1);        //that white's got to go later..
  RotVel[0]=0;              //we ain't spinnin right now
  RotVel[1]=0; 
  RotVel[2]=0;
  thrust=10;               //your Vegan Shipyards XT5000 Plasma Jet Propulsion System gives an acceleration of 10 m/s^2
  GunPos[0]=0;             //Oh, THIS is the gun
  GunPos[1]=0;
  GunPos[2]=-1.75;
  RadarPos[0]=1;           //Ah, put the radar over there
  RadarPos[1]=0;
  RadarPos[2]=0;
//  Vel[0]=25;             //If we're affected by gravity, we'd better start out in orbit
  scale=4.;                //original ship was too big
}

/****************Destructor*************/

Ship::~Ship(){}

/****************************************/

void Ship::Destroy()
{
	this->~Ship();
}

/****************************************/
//This draw function is complicated, so bear with me...
//p[19][3] stores the 19 points which make up the ship
//f[34][3] stores which points make up each of the 34 triangles needed to draw the ship
//The point data is static, and then is rotated every draw cycle
//Triangles are specified with a left-hand rule, as are normals.  Just to make things complicated
//Since both are left-handed, it gives the answer that GL expects (I think)
void Ship::Draw()
{

  float p[19][3];  //points
  int f[34][3]={{1,2,3},{1,3,4},{1,4,5},{1,5,6},              //DATA ENTRY  (NOTE: should probably be a private variable,
                {6,5,7},{6,7,8},{7,5,8},{9,6,8},{8,5,9},                     //but it isn't)
                {10,6,9},{11,10,9},{11,9,17},{9,5,17},{5,4,18},
                {10,11,12},{10,12,13},{1,6,10},{1,10,13},{1,13,2},
		{13,12,14},{2,13,14},{14,12,17},{14,17,3},{4,3,19},
		{2,14,15},{3,15,14},{2,15,16},{3,16,15},{2,16,3},{17,12,11},
                {5,18,17},{4,17,18},{3,17,19},{4,19,17}};
  int i;
  float j;

  float n[3]={0,0,0};        //This will hold the normal vector of each face
  float v1[3]={0,0,0};       //these are temp vectors which hold the rotated information
  float v2[3]={0,0,0};
  float Rcenter[3]={0,0,0};  //this will hold the rotated radar center (so the radar will stay in the same
                             //location relative to the ship

  float a[3][3]={{0,0,-.4},{.1,0,-.3},{-.1,0,-.3}};   //Arrow points for the direction pointer

  VECT(0,-.2,1,p[0]);        //MORE DATA ENTRY
  VECT(.4,-.1,1,p[1]);
  VECT(.4,.2,1,p[2]);
  VECT(0,.3,1,p[3]);
  VECT(-.4,.2,1,p[4]);
  VECT(-.4,-.1,1,p[5]);
  VECT(-.5,0,1.5,p[6]);
  VECT(-1.8,-.5,1,p[7]);
  VECT(-.2,0,-.3,p[8]);
  VECT(-.1,-.05,-.7,p[9]);
  VECT(-.1,0,-1.4,p[10]);
  VECT(.1,0,-1.4,p[11]);
  VECT(.1,-.05,-.7,p[12]);
  VECT(.2,0,-.3,p[13]);
  VECT(1.8,-.5,1,p[14]);
  VECT(.5,0,1.5,p[15]);
  VECT(0,.15,-.1,p[16]);
  VECT(-.8,.5,1.5,p[17]);
  VECT(.8,.5,1.5,p[18]);
  
  
  for (i=0;i<19;i++)    //ROTATE THOSE POINTS!
    {
    LINEUP(Orient[0],Orient[1],Orient[2],p[i],v1);   //Orient will point them in the right location
    ROTATE(RotAng[0],RotAng[1],RotAng[2],v1,v2);     //Any Rotation angle which might exist (not used, but I'm 
                                                     //nervous about taking it out)
    p[i][0]=v2[0]/scale;     //scale 'em
    p[i][1]=v2[1]/scale;
    p[i][2]=v2[2]/scale;
    }
  LINEUP(Orient[0],Orient[1],Orient[2],RadarPos,v1);   //do the same thing to the radar
  ROTATE(RotAng[0],RotAng[1],RotAng[2],v1,Rcenter);
  for (i=0;i<3;i++)                                   // and the direction arrow
    {
    LINEUP(Orient[0],Orient[1],Orient[2],a[i],v1);
    ROTATE(RotAng[0],RotAng[1],RotAng[2],v1,a[i]);
    }
     


  glPushMatrix();
  glTranslatef(Pos[0],Pos[1],Pos[2]);              //Put the ship... umm... here.  No, here...
                                                   //No, I liked it better over there...
  glShadeModel(GL_SMOOTH);                         //"smooth.. smooth.. <smack> smooth.."
  glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,color);
  glMaterialfv(GL_FRONT,GL_SPECULAR,white);   //make it nice and shiny
  glMaterialf(GL_FRONT,GL_SHININESS,64);
  
 
  for (i=0;i<34;i++)    // for every face...
    {
    if (i<4)  //(oh yeah, the first four triangles are the engine)
      glMaterialfv(GL_FRONT,GL_EMISSION,yellow);
    if (i==4) //no more engines
      glMaterialfv(GL_FRONT,GL_EMISSION,black);
    glBegin(GL_TRIANGLES);                         //Yes, these ARE triangles
    NORMAL(&p[f[i][0]-1][0],&p[f[i][1]-1][0],&p[f[i][2]-1][0],n);   //Get the normal vector (from functions.h)
    glNormal3fv(n);                         //here it is
    glVertex3fv(p[f[i][0]-1]);             //and here are the points
    glVertex3fv(p[f[i][1]-1]);
    glVertex3fv(p[f[i][2]-1]);
    glEnd();
    }
  
  glMaterialfv(GL_FRONT,GL_SPECULAR,black);   //not everything is shiny, ya know...
  glMaterialf(GL_FRONT,GL_SHININESS,0);

  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,green);        //the radar should be green... definitly green
  glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,black);
    glPointSize(4.0);
  glEnable(GL_POINT_SMOOTH);

  for (j=0;j<(2*3.14159);j+=(3.14159/20.0))    //a whole bunch of angles from 0 to 2*PI
    {
    glBegin(GL_POINTS);
    glVertex3f(-sin(j)/2.+Rcenter[0],Rcenter[1],-cos(j)/2.+Rcenter[2]);  // make three circles to get a cool 3D effect
    glVertex3f(Rcenter[0],sin(j)/2.+Rcenter[1],-cos(j)/2.+Rcenter[2]);
    glVertex3f(cos(j)/2.+Rcenter[0],sin(j)/2.+Rcenter[1],Rcenter[2]);
    glEnd();
    }

  glDisable (GL_POINT_SMOOTH);
  glPointSize(1);
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,cyan);    //Back in my day, we only had 8 colors.. and we LIKED it!
  glLineWidth(2.0);
  glBegin(GL_LINE_STRIP);                              //This is for the direction arrow
  glVertex3f(Rcenter[0],Rcenter[1],Rcenter[2]);               // put that arrow here
  glVertex3f(Rcenter[0]+a[0][0],Rcenter[1]+a[0][1],Rcenter[2]+a[0][2]);  
  glVertex3f(Rcenter[0]+a[1][0],Rcenter[1]+a[1][1],Rcenter[2]+a[1][2]);
  glVertex3f(Rcenter[0]+a[0][0],Rcenter[1]+a[0][1],Rcenter[2]+a[0][2]);
  glVertex3f(Rcenter[0]+a[2][0],Rcenter[1]+a[2][1],Rcenter[2]+a[2][2]);
  glEnd();

  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,magenta);  // IT'S PURPLE, DAMMIT!!!!!
  glBegin(GL_LINES);
  glVertex3f(Rcenter[0],Rcenter[1],Rcenter[2]);          //Just make a line.. another arrow would give me a headache
  glVertex3f(Rcenter[0]+Vel[0]/50.,Rcenter[1]+Vel[1]/50.,Rcenter[2]+Vel[2]/50.);   //make it big if we's goin fast
  glEnd();
  glLineWidth(1);
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,black);
  
  glPopMatrix();
}   //IT'S DONE!!!!!

/****************************************/

void Ship::FileWrite()
{

}

/****************************************/
// I dunno... 10 sounds good....

float Ship::GetMass()
{
  return (10);
}

/*****************************************/

void Ship::SetColor(float r, float g, float b, float a)
{
  color[0]=r;
  color[1]=g;
  color[2]=b;
  color[3]=a;
}

/******************************************/

void Ship::SetColor(float c[])
{
  color[0]=c[0];
  color[1]=c[1];
  color[2]=c[2];
  color[3]=c[3];
}

/*******************************************/
//  Hit the gas!
void Ship::CheckThrust ()
{
  float wandFront[3];

  CAVEGetVector(CAVE_WAND_FRONT,wandFront);    //where was that wand?

  if (CAVEBUTTON3)                             //remember, the gas is the one on the RIGHT!
    {
      Acc[0]+=wandFront[0]*thrust;             //acceleration components
      Acc[1]+=wandFront[1]*thrust;
      Acc[2]+=wandFront[2]*thrust;
    }
}

/********************************************/
//  Yes, the ship does create bullets, so return a 1
//  Gives the current gun position (relative to ship center)
//  and ship velocity (so we don't run into our bullets)
int Ship::GetBulletData(float locate[], float velocity[])
{
  locate[0]=GunPos[0]/scale;  //Muzzle exit location
  locate[1]=GunPos[1]/scale;
  locate[2]=GunPos[2]/scale;
  float temp[3];               //TEMP
  LINEUP(this->Orient[0],this->Orient[1],this->Orient[2],locate,temp); //rotate that sucker (functions.h) so the gun
  ROTATE(this->RotAng[0],this->RotAng[1],this->RotAng[2],temp,locate); // faces the same way the ship does (nice feature)
  velocity[0]=Vel[0];          //only sub-light speeds, please.
  velocity[1]=Vel[1];
  velocity[2]=Vel[2];
  return(1);
}

/*********************************************/
//  Say your prayers...
//
//  Actually, this Explode function should never be called, since a collision with the ship decreases shields, and
//  a zero shield will restart the game.
Object * Ship::Explode()
{
  fprintf(stderr,"SHIP EXPLODE????\n\n\n\n\n\n\n\n\n");            //you did WHAT?
  fprintf(stderr,"SHIP POS %f %f %f\n\n\n",Pos[0],Pos[1],Pos[2]);  //here is was
  return(NULL);
}

/**************************************************/
//  Where's the radar on this thing?  Oh, right next to the cigarette lighter.
void Ship::GetRadarPos(float rpos[])
{
  float v1[3];   //TEMP
  LINEUP(Orient[0],Orient[1],Orient[2],RadarPos,v1);  // Rotate it, of course
  ROTATE(RotAng[0],RotAng[1],RotAng[2],v1,rpos);
  rpos[0]+=Pos[0];   // and make sure it's NEXT to the ship
  rpos[1]+=Pos[1];
  rpos[2]+=Pos[2];
}
  

#endif
