#include "object.h"

#ifndef SPHERE_H
#define SPHERE_H
extern GLUquadricObj *sphereObj;

#define NUMPHI 16
#define NUMTHETA 16

/*************Class Declaration**************/

class Sphere:public Object{
private:
	float radius;
	float color[4];
	float density;
	//	float points[2 + (NUMPHI-2)*NUMTHETA];
	//        float tris[2*NUMTHETA + 2*(NUMTHETA)*(NUMPHI-3)][3];
	//	float norms[2*NUMTHETA + 2*(NUMTHETA)*(NUMPHI-3)][3];
public:
	Sphere(float r,float d);
	Sphere(float r);
	Sphere();
	~Sphere();
	void Destroy();
	void Draw();	
	void FileWrite();
	float GetMass();
	void SetColor(float r, float g, float b, float a);
	void SetColor(float c[]);
	void SetVel(float v[]);
	void SetVel(float vx,float vy, float vz);
	Object *Explode();
	int TrashIt(){return 0;};
	int Points(){return ((int)(5/radius));};
};

/****************Contructor*************/

Sphere::Sphere(float r,float d):Object()
{
	radius=r;
	density=d;
	this->SetColor(1,1,1,1);
}

/****************Contructor*************/

Sphere::Sphere(float r):Object()
{
	radius=r;
	density=10;
	Vel[0]=(float)rand()/32767*3;
	Vel[1]=(float)rand()/32767*3;
	Vel[2]=(float)rand()/32767*3;
	this->SetColor(1,1,1,1);
}

/*********Default Constructor***********/

Sphere::Sphere():Object()
{
  radius=1;
  density=10;
  this->SetColor(1,1,1,1);
}

/****************Destructor*************/

Sphere::~Sphere(){}

/****************************************/

void Sphere::Destroy()
{
	this->~Sphere();
}

/****************************************/

void Sphere::Draw()
{
        glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,color);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,gray);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,24);
	glPushMatrix();
	glTranslatef(Pos[0],Pos[1],Pos[2]);
	if (radius > 1)
	  gluSphere(sphereObj,radius,32,32);   //used to be 8,8
	else if (radius > .5)
          gluSphere(sphereObj,radius,16,16);
	else
	  gluSphere(sphereObj,radius,8,8);
	glPopMatrix();
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,0);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,black);
}

/****************************************/

void Sphere::FileWrite()
{

}

/****************************************/

float Sphere::GetMass()
{
  return (density*CUBE(radius)*3/4);
}

/*****************************************/

void Sphere::SetColor(float r, float g, float b, float a)
{
  color[0]=r;
  color[1]=g;
  color[2]=b;
  color[3]=a;
}

/******************************************/

void Sphere::SetColor(float c[])
{
  color[0]=c[0];
  color[1]=c[1];
  color[2]=c[2];
  color[3]=c[3];
}

/*****************************************/

void Sphere::SetVel(float v[])
{
  Vel[0]=v[0];
  Vel[1]=v[1];
  Vel[2]=v[2];
}

/*****************************************/

void Sphere::SetVel(float vx, float vy, float vz)
{
  Vel[0]=vx;
  Vel[1]=vy;
  Vel[2]=vz;
}

/*******************************************/

Object *Sphere::Explode()
{
  Sphere *ball1;
  Sphere *ball2;
  Sphere *ball3;
  Sphere *ball4;

  if (this->radius > .5)
    {
      //fprintf(stderr,"killing sphere\n");
      //    SharedMemArena::on();
    ball1 = new Sphere(this->radius/2.5,density*CUBE(2.5)/4);
    ball2 = new Sphere(this->radius/2.5,density*CUBE(2.5)/4);
    ball3 = new Sphere(this->radius/2.5,density*CUBE(2.5)/4);
    ball4 = new Sphere(this->radius/2.5,density*CUBE(2.5)/4);
    //fprintf(stderr,"got new spheres\n");
    //    SharedMemArena::off();
    if (!ball1||!ball2||!ball3||!ball4)
      {
      fprintf(stderr,"OUT OF MEMORY!\n");
      return(NULL);
      }
    ball1->SetColor(this->color);
    ball2->SetColor(this->color);
    ball3->SetColor(this->color);
    ball4->SetColor(this->color);
  
    ball1->Move(this->Pos[0],this->Pos[1]+sqrt(2)*this->radius/3,this->Pos[2]-sqrt(2)*this->radius/3);
    ball2->Move(this->Pos[0],this->Pos[1]-sqrt(2)*this->radius/3,this->Pos[2]-sqrt(2)*this->radius/3);
    ball3->Move(this->Pos[0]+sqrt(2)*this->radius/3,this->Pos[1],this->Pos[2]+sqrt(2)*this->radius/3);
    ball4->Move(this->Pos[0]-sqrt(2)*this->radius/3,this->Pos[1],this->Pos[2]+sqrt(2)*this->radius/3);
    //fprintf(stderr,"spheres moved\n");

    ball1->SetVel(this->Vel[0],this->Vel[1]+sqrt(2)/10,this->Vel[2]-sqrt(2)/10);
    ball2->SetVel(this->Vel[0],this->Vel[1]-sqrt(2)/10,this->Vel[2]-sqrt(2)/10);
    ball3->SetVel(this->Vel[0]+sqrt(2)/10,this->Vel[1],this->Vel[2]+sqrt(2)/10);
    ball4->SetVel(this->Vel[0]-sqrt(2)/10,this->Vel[1],this->Vel[2]+sqrt(2)/10);
  
    ball1->SetRadiusOfCollision(this->radius/2.5);
    ball2->SetRadiusOfCollision(this->radius/2.5);
    ball3->SetRadiusOfCollision(this->radius/2.5);
    ball4->SetRadiusOfCollision(this->radius/2.5);
    //fprintf(stderr,"collision set\n");
    /*
    ball1->TurnGravOff();
    ball2->TurnGravOff();
    ball3->TurnGravOff();
    ball4->TurnGravOff();
    */
    /*  
    ball1->Link(list->Next());
    list->Link((Object *)ball1);
    ball2->Link(list->Next());
    list->Link((Object *)ball2);
    ball3->Link(list->Next());
    list->Link((Object *)ball3);
    ball4->Link(list->Next());
    list->Link((Object *)ball4);
    */
    ball1->Link((Object *)ball2);
    ball2->Link((Object *)ball3);
    ball3->Link((Object *)ball4);
    ball4->Link(NULL);
    //fprintf(stderr,"linked\n");
    }
  else ball1=NULL;
  //  this->Destroy();
  return(ball1);
}  

#endif
