#include <cave_ogl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <unistd.h>
#include "our_colors.h"
#include "shared.h"
#include "object.h"
#include "head.h"
//#include "sphere.h"
#include "ship.h"
#include "bullet.h"
#include "bomb.h"
#include "functions.h"
#include "shield.h"
#include "planet.h"
#include "badguy.h"
#include <math.h>
#include "nums.c"

#ifndef SYSTEM_H             //if not previously defined
#define SYSTEM_H             //mark as defined

#define MAINMENU 0           //states of the game
#define PLACEASTEROIDS 1     //starting position of the game
#define PLACESHIP 2          //everything else done now just place your ship
#define PLAYGAME 3           //time to dance
#define GAMEOVER 4           //you died....again?!?!?!

extern GLubyte star_tex[TEXSIZEX][TEXSIZEY][4];   //texture for starfield
extern GLuint texName;

class System{
 protected:
  float dt;                  //amount of time per tick
  int gamestatus;            //what is this player fool doing now???
  Head *ObjectList;          //Beginning of our glorious list of objects
//  float stars[500][3];       //The universe, as created by Prong
  int soundhandle;           //handle to our sound process
  Shield *EShield;           //our ship has a cool shield
  float time;                //time that has expired
  long score;                //how many points so far
  int StartAsteroids;        //number of asteroids to start play with
  int StartBadGuys;          //number of bad guys to start against
  int AsteroidRefresh;       //when are new asteroids created
  int BadGuyRefresh;         //when do bad guys come to die
  float star_rot[3];         //rotation angle of starfield

 public:
  System();                  //one and only (default) constructor
  ~System();                 //default destructor
  void SetDT(float newdt){dt=newdt;}  //set time per tick
  void InitWorld();          //set up objects in our universe
  void KillSound();          //stop that infernal racket
  void DrawWorld();          //used for drawing process
  void DestroyWorld();       //and Prong said let there be darkness
  void Run();                //Run one Tick
  void MainMenu();           //Not implemented ....yet
  void PlaceAsteroids();     //Put some furniture in the empty house
  void PlaceShip();          //wait for button one and two to place ship
  void PlayGame();           //interpret our input
  void GameOver();           //you lose so sorry... Tell them what they've won
  void AddAsteroid(float x, float y, float z);  //place asteroid around location x,y,z
  void AddBadGuy(float x, float y, float z);    //place bad guy around locatino x,y,z
  void AddPlanet();                             //put planet at predefined position
  void CreateBullet(float loc[],float vel[]);   //put bullet relative to our ship
  void CreateBadBullet(float loc[],float vel[]);//put bullet relative to other object
  void CreateBomb();                            //put bomb relative to our ship
  void CleanUp();                               //destroys bullets that are past their prime
  void ModifyAccel();                           //Mr. Newtons gravity kicks in
  int ResolveCollisions();   //returns score change    //Something went bump in the night
  void ModifyPos();                             //according to their velocity, this changes every objects position
  void ModifyRotation();                        //rotate objects in list
  void InsertObject(Object *place,Object *thing);   //we have a new object to wait in line
  Object *RemoveObject(Object *place);              //you're out of here
  void DrawRadar(Object *thing,float Rpos[],float Fpos[]);  //draw that beautiful radar
  void InitStarField();                             //make that universe Prong
  void DrawStarField();                             //draw the thing too
  void DrawScore();
};

/////////////////////////////////////////////////////
//Constructor//
System::System()
{
  ObjectList = new Head();            //linked list head
  gamestatus=1;                       //Start at the Asteroid
  this->SetDT(.005);                   //default dt is set
  EShield=NULL;                       //There is no ship or shield to draw
  star_rot[0]=0;
  star_rot[1]=0;
  star_rot[2]=0;
  score=0;
}

//////////////////////////////////////////////////////
//Destructor//

System::~System()                  
{
  this->DestroyWorld();               //destory all
}

/////////////////////////////////////////////////////
///sets constants for world////
void System::InitWorld()
{
#ifdef SOUND_ON
  soundhandle = AUDinit("game.aud");     //let there be sound...and there was... and it was good
#endif SOUND_ON
  InitStarField();                       //make a star field for the almighty Pancake
  StartAsteroids=5;                      //start them off easy
  StartBadGuys=0;                        //no baddies to hunt you down AND KILL YOU LIKE A DOG
  AsteroidRefresh=500;                   //every 500 ticks
  BadGuyRefresh=8000;                    //every 8000 ticks
  this->score = 0;                       //you have no points yet
  star_rot[0]=0;
  star_rot[1]=0;
  star_rot[2]=0;
}

/////////////////////////////////////////////////////
///Used by feedback process to draw our pretty picture
void System::DrawWorld()
{
 Object *current;                       //for traversing our object list
 Ship *fighter;                         //our killa ship
 float Fpos[3]={0,0,0};                 //Ship position
 float Rpos[3]={0,0,0};                 //Radar Position
 float Lpos[4]={1,0,1,0};               //Light Position (fourth value scales... 
					//    (x,y,z,w)= point (x/w,y/w,z/w)... light at infinity)
 float Ldir[3]={-1,0,-1};               //Direciton light is pointing
 int i;
 glClearColor(0., 0., 0., 0.);          //Prong said DARK!!!!!!!!!!
 glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
 glLightModelfv(GL_LIGHT_MODEL_AMBIENT,darkgray);
 glLightfv(GL_LIGHT1,GL_POSITION,Lpos);
 glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION,Ldir);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,white);
 glLightfv(GL_LIGHT1,GL_AMBIENT,black);
 glLightfv(GL_LIGHT1,GL_SPECULAR,gray);
 glEnable(GL_LIGHTING);
 glDisable(GL_LIGHT0);
 DrawStarField();                               //draw background stuff
 if (gamestatus==PLAYGAME)                      //if we're playing
   {
   fighter=(Ship *)ObjectList->Next();          //Access our ship (at head of the list)
   fighter->GetRadarPos(Rpos);                  //Where should that perty radar be any hoo??
   Fpos[0]=fighter->GetX();                     //get fighter position
   Fpos[1]=fighter->GetY();
   Fpos[2]=fighter->GetZ();
   }
 glEnable(GL_LIGHT1);
 DrawScore();


 for (current = ObjectList->Next(); current; current = current->Next())
   {
     if (!current->IsKilling())
       {
       while(current->IsLocked()){}            //if our object is not being removed 
       
       current->Lock();                   //lock object   
       current->Draw();                   //draw the sucka
       current->UnLock();                 //remove lock from object
       }
  
//     else fprintf(stderr,"It was Locked (from Draw)\n");
   }

 glDisable(GL_LIGHT1);
 
 /*
 glPushMatrix();
 temp=CAVEFar;
 */
 CAVEFar=1000;                             //how far out you can see
 /*
 glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,white);
 glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,white);
 glTranslatef(500,0,500);
 gluSphere(sphereObj2,10,16,16);
 glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,black);
 CAVEFar=temp;
 glPopMatrix();
 */

 if (gamestatus==PLAYGAME)                  //if our ship is on the screen
   for (current = ObjectList->Next(); current; current = current->Next())
     {
     DrawRadar(current,Rpos,Fpos);          //draw each object inn the radar
     }
  
 if (EShield)                               //if our shield should be drawn
   {
   if (!EShield->IsKilling())
     {
     while (EShield->IsLocked()){}          //and shield is not going bye-bye
     EShield->Lock();                       //lock it
     EShield->Draw();                       //draw it
     EShield->UnLock();                     //unlock it 
     }
   }
 glDisable(GL_LIGHTING);
}

/////////////////////////////////////////////////////
//we are done, baby//////////////////////////////////
void System::DestroyWorld()
{
  Object *temp;                         //traversal objects
  Object *loop;
  for(loop=ObjectList;loop->Next();)    //go to every object in the list
    {
      while (loop->Next()->IsLocked()) {sginap(1);}    //wait for object to be free
      loop->Next()->Killing();                  //lock object 
      temp=RemoveObject(loop);               //destroy the object from list
      temp->Destroy();                       //crush object like a grape
    }
}

/////////////////////////////////////////////////////
////What do we do////////////////////////////////////
////Our program goes here to find out////////////////
void System::Run()
{
  switch (gamestatus)
   {
     case MAINMENU:   //MAIN MENU
       {
	 this->MainMenu();
	 break;
       }
     case PLACEASTEROIDS:   //Placing Asteroids
       {
	 this->PlaceAsteroids();
	 break;
       }
     case PLACESHIP:   //Placing the Ship
       {
	 this->PlaceShip();
	 break;
       }
     case PLAYGAME:   //Play the Game
       {
	 this->PlayGame();
	 break;
       }
     case GAMEOVER:   //Game just over
       {
	 this->GameOver();
	 break;
       }
   }
}

/////////////////////////////////////////////////////
/////////Not yet implemented
void System::MainMenu()
{

}

/////////////////////////////////////////////////////
////sets up our basic universe
void System::PlaceAsteroids()
{
 for (int loop=0;loop<StartAsteroids;loop++)
   {
     this->AddAsteroid(0,0,-60);             //makes our predetermined # of asteroid around the planet   
   }
 for (loop=0;loop<StartBadGuys;loop++)
   {
     this->AddBadGuy(0,0,-60);               //makes our badguys around planet
   }
 this->AddPlanet();                          //put the planet smack dab in the middle of the fray

 StartAsteroids+=2;                          //for next round
 StartBadGuys+=1;                            
 AsteroidRefresh=AsteroidRefresh;            //asteroid regeneration rate not modified (but could be later)
 BadGuyRefresh=BadGuyRefresh/2;              //more bad guys faster
 gamestatus=PLACESHIP;                       //Mr. DeMill, I'm ready for my close up
}

/////////////////////////////////////////////////////
///waits for the user to press and release the left wand button
void System::PlaceShip()
{
  if (CAVEButtonChange(1)==-1)
    {
      Ship *fighter;                                 ///pointer to fighter
      float wandPos[3],wandFront[3],wandOrient[3];   ///temporary pointers
    
                  ///The only reason for three '/' is to make Prong (Pranas) angry.

      fighter = new Ship();                          ///make a ship
  
      if (!fighter)                                  //check to see if we actually did it
	{
	  fprintf(stderr,"OUT OF MEMORY!\n");
	  return;
	}
      fighter->SetColor(gray);                       //our fighter is a stylish gray

      CAVEGetPosition(CAVE_WAND,wandPos);            //where to put it
      CAVEGetVector(CAVE_WAND_FRONT,wandFront);      //where is it pointing
      CAVEGetOrientation(CAVE_WAND,wandOrient);      //is it twisted
      wandOrient[0]*=PI/180;
      wandOrient[1]*=PI/180;
      wandOrient[2]*=PI/180;
      fighter->Move(wandPos[0] + wandFront[0], wandPos[1] + wandFront[1],
		    wandPos[2] + wandFront[2]);      //put the ship where it belongs
      fighter->SetOrient(wandOrient[0],wandOrient[1],wandOrient[2]); 
      fighter->TurnGravOff();                        //no ship gravity
      InsertObject(ObjectList,(Object *)fighter);    //put ship in our wonderful list
      fighter->SetRadiusOfCollision(.5);             //when does our ship actually get hit.
     
      Shield *thing;                                 //we have to have protection

      thing = new Shield();                          //instantiate object
      thing->Move(wandPos[0] + wandFront[0], wandPos[1] + wandFront[1],
		    wandPos[2] + wandFront[2]);
      thing->SetRadiusOfCollision(-100);             //can't collide with anything
      EShield=thing;                                 //use Eshield variable
      gamestatus=PLAYGAME;                           //Let's get ready to rumble
      this->time=0.0;                                //Judges, start your watches
    }
}

/////////////////////////////////////////////////////
////The Function Runs the Heart of the Game//////////
void System::PlayGame()
{
  Object *current;
  float shipx, shipy, shipz;
  float loc[3],vel[3];
  
  int timeint;

  this->CleanUp();                           //maybe take care of in modifypos
  this->ModifyAccel();                       //gravity
  score += this->ResolveCollisions();        //did I get it??????
  for (current = ObjectList->Next(); current; current = current->Next())
     current->Tick(dt);                      //the world moves on (book reference)
  if (EShield)                               //if there is still a shield
    EShield->Tick(dt);                       //Tick that sucka too
  this->time+=dt;                            //increment time
  timeint=(int)(this->time/dt);              //how many intervals have gone by
  if ((timeint%AsteroidRefresh==0)&&(*CAVEFramesPerSecond>12.))
    {
     shipx=ObjectList->Next()->GetX();       //frame rate fast enough to add another asteroid
     shipy=ObjectList->Next()->GetY();
     shipz=ObjectList->Next()->GetZ();
     this->AddAsteroid(shipx,shipy,shipz); 
     fprintf(stderr,"Asteroid added at time = %f, frame rate = %f\n",this->time,*CAVEFramesPerSecond);
    }
  if ((timeint%BadGuyRefresh==0)&&(*CAVEFramesPerSecond>12.))
    {
     shipx=ObjectList->Next()->GetX();       //frame rate fast enough to add another bad guy
     shipy=ObjectList->Next()->GetY();
     shipz=ObjectList->Next()->GetZ();
     this->AddBadGuy(shipx,shipy,shipz); 
     fprintf(stderr,"BadGuy added at time = %f, frame rate = %f\n",this->time,*CAVEFramesPerSecond);
    }

  this->ModifyPos();                         //moves thing relative to the ship
  this->ModifyRotation();                    //enemy ships point to us and the world spins if needed

  if ((CAVEBUTTON2==1)&&(timeint%3==0))   /* THIS IS FOR MACHINE-GUN.  */
//  if (CAVEButtonChange(1)==1)
    {
      ObjectList->Next()->GetBulletData(loc,vel);     //shoooooooooot
      this->CreateBullet(loc,vel);
#ifdef SOUND_ON	       
      AUDupdate(soundhandle,"Bullet",0,NULL);
#endif SOUND_ON
    }
  /*     //no bombs for now (memory problems... see ResolveCollisions)
  if ((CAVEBUTTON2==1)&&(timeint%6==0))
//  if (CAVEButtonChange(2)==1)
    {
      this->CreateBomb();                             ///nuke 'em
    }
    */

  for (current=ObjectList->Next()->Next();current;current=current->Next())
    {
      if (current->GetBulletData(loc,vel))               ///if it's a bad guy
	if (current->DistanceToObject(ObjectList->Next())<30)     //if their close enough to shoot
	  if (timeint%3==0)
	    CreateBadBullet(loc,vel);                             //shoot every 3 ticks
    }

  if (!(ObjectList->Next()->Next()))                  //only a ship
    {
    DestroyWorld();                                   //destroy everything
    while (EShield->IsLocked()) {sginap(1);}                    //get rid of shield
    EShield->Killing();                                  //lock shield 
    EShield->Destroy();                               //wreck the shield object
    EShield=NULL;                                     //no more shield....*sniff**sniff*
    gamestatus=PLACEASTEROIDS;
    }
  else if (EShield->GetEnergy()<.05)                  //shield power gone
    {
    StartAsteroids=5;                                 //reset conditions
    StartBadGuys=0;
    AsteroidRefresh=500;
    BadGuyRefresh=8000;
    fprintf(stderr,"\n\nYOUR SCORE:  %d\n\n",this->score);     //print final score
    this->score=0;                                    //reset score
    gamestatus = GAMEOVER;                            //Your Game is Over Chump
    }
  else if (this->time>40)                             //You survived this round, highlander
    {
    fprintf(stderr,"\ncurrent score:  %d\n",this->score);
    gamestatus = GAMEOVER;
    }
  
  //fprintf(stderr,"CAVE time = %f\n",*CAVETime);
  /*
  GLfloat sizes[2];
  GLfloat step;
  GLfloat fsize;

  glGetFloatv(GL_POINT_SIZE,&fsize);
  glGetFloatv(GL_POINT_SIZE_RANGE,sizes);
  glGetFloatv(GL_POINT_SIZE_GRANULARITY,&step);

  fprintf(stderr,"range = %f , %f\n",sizes[0],sizes[1]);
  fprintf(stderr,"step = %f    size = %f\n",step,fsize);
  */  
}

/////////////////////////////////////////////////////
////Give it up loser///////
void System::GameOver()
{
 DestroyWorld();                                      //Destroy All
 while (EShield->IsLocked()) {sginap(1);}                       //Wreck Shield
 EShield->Killing();
 EShield->Destroy();
 EShield=NULL;
 gamestatus=PLACEASTEROIDS;                           //Next Round of Play

}

/////////////////////////////////////////////////////
//add asteroid around position x,y,z
void System::AddAsteroid(float x,float y, float z)
{
  Sphere *ball;
  float Pos[3];
  Pos[0]=(float)rand()/32767*120-60;                     //get random x, y, and z
  Pos[1]=(float)rand()/32767*120-60;
  Pos[2]=(float)rand()/32767*120-60;
  if (abs(Pos[0]<5))                                     //if too close to ship, move it away
      Pos[0]+=5;
  if (abs(Pos[1]<5))                                     //if too close to ship, move it away
      Pos[1]+=5;
  if (abs(Pos[2]<5))                                     //if too close to ship, move it away
      Pos[2]+=5;
  ball = new Sphere(2);                                  //make asteroid
 

  if (!ball)                                             //did we actually make it or not
    {
      fprintf(stderr,"OUT OF MEMORY!\n");
      return;
    }
  
 float Color[4];                                         //set up color vector
 Color[0]=(float)rand()/65535;                           //random color
 Color[1]=(float)rand()/65535;
 Color[2]=(float)rand()/65535;
 Color[3]=(float)rand()/65535;
 ball->SetColor(Color[0],Color[1],Color[2],Color[3]);    //set color
 ball->Move(x + Pos[0], y + Pos[1], z + Pos[2]);         //move asteroid to correct location
 if (ObjectList->Next())                                 //if ship exists
   InsertObject (ObjectList->Next(),(Object *)ball);     //put object in list after ship
 else 
   InsertObject(ObjectList,(Object *)ball);              //put object in list
 ball->SetRadiusOfCollision(2.);                         //set up radius of collision for object
}

/////////////////////////////////////////////////////
/////Add an evil sucka around x,y,z//////////////////
void System::AddBadGuy(float x, float y, float z)
{
  Badguy *ball;                              //bad guy pointer
  float Pos[3];   
  Pos[0]=(float)rand()/32767*120-60;         //get random position around ship
  Pos[1]=(float)rand()/32767*120-60;
  Pos[2]=(float)rand()/32767*120-60;

  ball = new Badguy();                       //make bad guy

  if (!ball)                                 //did we get to make a bad guy
    {
      fprintf(stderr,"OUT OF MEMORY!\n");
      return;
    }
  
 ball->Move(x + Pos[0], y + Pos[1], z + Pos[2]);      //move bad guy to correct location
 if (ObjectList->Next())                              //if ship exists
    InsertObject (ObjectList->Next(),(Object *)ball); //put bad guy in after ship
 else 
    InsertObject(ObjectList,(Object *)ball);          //put bad guy in list
 ball->SetRadiusOfCollision(.5);                      //radius of collision is small
}


/////////////////////////////////////////////////////
////put in a big sphere with large mass
void System::AddPlanet()
{
  Planet *rock;                     //pointer to planet
  rock=new Planet(10);              //instantiate, Number One
  rock->Move(0,0,-30);              //Put it in its place
  InsertObject(ObjectList,(Object *)rock);           //Put in its place in the list
  rock->SetRadiusOfCollision(10);                    //It's pretty big
  rock->MakeUnExplodable();                          //Nobody bother me...
}
  

/////////////////////////////////////////////////////
///create a bullet with a given location and velocity vector
void System::CreateBullet(float loc[],float vel[])
{
  Bullet *thing;                            //bullet pointer
  Ship *fighter;

  fighter=(Ship *)ObjectList->Next();       //grab the ship at the front of the list

  thing=new Bullet(vel);                    //Make a bullet 
  
  thing->MoveRel(loc[0]*2,loc[1]*2,loc[2]*2);  //Put it in the right spot
  InsertObject(fighter,thing);                 //Put bullet in the list

}

/////////////////////////////////////////////////////
///Bad Guys make bullets too
void System::CreateBadBullet(float loc[],float vel[])
{
  Bullet *thing;                            //Bullet pointer
  Ship *fighter;

  fighter=(Ship *)ObjectList->Next();

  thing=new Bullet(50,vel);                 //bullet has smaller range than ours
  
  thing->MoveRel(loc[0],loc[1],loc[2]);     //put bullet in its place
  InsertObject(fighter,thing);              //put in the list

}

/////////////////////////////////////////////////////
//Make a bomb sucka
void System::CreateBomb()
{
  Bomb *thing;                              //Bomb pointer
  Ship *fighter;
  float v[3]={0,0,0};
  float p[3]={0,0,0};

  fighter=(Ship *)ObjectList->Next();       //Get ship info
  fighter->GetBulletData(p,v);

  thing=new Bomb(v);                        //make the bomb
  
  thing->MoveRel(p[0]*2,p[1]*2,p[2]*2);     //put that bomb in its place
  InsertObject(fighter,thing);              //attach to list
}

/////////////////////////////////////////////////////
////Destroy bullets that need to go away
void System::CleanUp()
{
 Object *current;
 Object *temp;
 current = (Object *)ObjectList;
 for (;current->Next();)                   //every obhect
   if (current->Next()->TrashIt())         //if bullet needs to die
     {
       while (current->Next()->IsLocked()) {}    //wait until not locked
       current->Next()->Killing();                  //lock it
       temp=RemoveObject(current);               //your out of there
       temp->Destroy();                          //bye
     }
   else current=current->Next();           //next object in list
}

/////////////////////////////////////////////////////
/////Find gravity reaction of all objects////////////
void System::ModifyAccel()
{
  Object *current,*loop;
  Ship *fighter;
  fighter=(Ship *)ObjectList->Next();
  
  for (current = ObjectList->Next(); current; current = current->Next())    //go through entire list
     {
       current->ResetAccel();                                     //start accel at zero
       if (current->IsGravOn())                                   ///if affected by gravity
         for (loop = ObjectList->Next(); loop; loop = loop->Next())   //go through the list for each object
	   if (loop->IsGravOn())                                  //if that object reacts to gravity
	     current->CalcAccel(loop);                            //calc accel due to another object
								  //in case reacting with itself (taken care of)
     }
  fighter->CheckThrust();                                         //check if we are accelerating
}

/////////////////////////////////////////////////////
////Rotates all ships to face our ship and takes care of turning the world when needed
void System::ModifyRotation()
{
  Object *current;
  float wandFront[3],wandOrient[3];
  float RotX,RotY,RotZ;
  float ShipX,ShipY,ShipZ;
  float ShipVel[3],NewShipVel[3];
  float ObjPos[3],NewObjPos[3];
  float theta,phi,norm;

  CAVEGetVector(CAVE_WAND_FRONT,wandFront);     //where is that pesky wand
  CAVEGetOrientation(CAVE_WAND,wandOrient);
  RotZ=wandOrient[2]*PI/18000;
  RotX=wandFront[0]*PI/200;
  RotY=wandFront[1]*PI/200;


  if ((fabs(RotX) > .0075)||(fabs(RotY) > .0075)||(fabs(RotZ) > .0075))
    {

      
    star_rot[0]+=RotX*((float)180)/PI;
    star_rot[1]-=RotY*((float)180)/PI;
//    star_rot[2]-=RotZ*((float)180)/PI;

//    fprintf(stderr,"%f, %f,   %f, %f\n",RotX,RotY,star_rot[0],star_rot[1]);
//    fprintf(stderr,"%f, %f,   %f, %f\n",RotX,RotY,RotX*((float)180)/PI,RotY*((float)180)/PI);
//    fprintf(stderr,"%f, %f,   %f, %f\n",RotX*((float)180)/PI,RotY*((float)180)/PI,star_rot[0],star_rot[1]);

//    ROTATE(-RotY,RotX,0,star_rot,star_rot);

    ShipX=ObjectList->Next()->GetX();          //get ship position
    ShipY=ObjectList->Next()->GetY();
    ShipZ=ObjectList->Next()->GetZ();

    ShipVel[0]=ObjectList->Next()->GetVelX();  //get ship velocity
    ShipVel[1]=ObjectList->Next()->GetVelY();
    ShipVel[2]=ObjectList->Next()->GetVelZ();
//    ROTATE(-RotY,RotX,-RotZ,ShipVel,NewShipVel);//rotate ship velocity vector according to wand orientation
    ROTATE(-RotY,RotX,0,ShipVel,NewShipVel);//rotate ship velocity vector according to wand orientation
    ObjectList->Next()->SetVel(NewShipVel[0],NewShipVel[1],NewShipVel[2]);   //reset new velocity

    for (current = ObjectList->Next()->Next(); current; current= current->Next())
      {
      ObjPos[0]=current->GetX();                //get object position
      ObjPos[1]=current->GetY(); 
      ObjPos[2]=current->GetZ();
      ObjPos[0]-=ShipX;                         //subtract ship position
      ObjPos[1]-=ShipY;
      ObjPos[2]-=ShipZ;
//      ROTATE(-RotY,RotX,-RotZ,ObjPos,NewObjPos); //rotate object about wand
      ROTATE(-RotY,RotX,0,ObjPos,NewObjPos); //rotate object about wand
      if (current->IsBadGuy())       //badguys point at us
	{
        theta=asinf(-(NewObjPos[0]/sqrt(SQR(NewObjPos[0])+SQR(NewObjPos[2]))));
        if (NewObjPos[2]<0)
	  theta=PI-theta;
	norm=sqrt(SQR(NewObjPos[0])+SQR(NewObjPos[1])+SQR(NewObjPos[2]));
        phi=asinf(NewObjPos[1]/norm);
        current->SetOrient(-phi,-theta,0);
	norm=norm*10;
	current->AddVel(-NewObjPos[0]/norm,-NewObjPos[1]/norm,-NewObjPos[2]/norm);
	}

      NewObjPos[0]+=ShipX;      //add back ship position
      NewObjPos[1]+=ShipY;
      NewObjPos[2]+=ShipZ;

      current->Move(NewObjPos[0],NewObjPos[1],NewObjPos[2]);    //move object in space to this position

      ShipVel[0]=current->GetVelX();                //alter velocity accordingly
      ShipVel[1]=current->GetVelY();
      ShipVel[2]=current->GetVelZ();
//      ROTATE(-RotY,RotX,-RotZ,ShipVel,NewShipVel);
      ROTATE(-RotY,RotX,0,ShipVel,NewShipVel);

      current->SetVel(NewShipVel[0],NewShipVel[1],NewShipVel[2]);

      }
    }
  else
    {
     //same as above just don't worry about all objects, just have ships point at us

    ShipX=ObjectList->Next()->GetX();
    ShipY=ObjectList->Next()->GetY();
    ShipZ=ObjectList->Next()->GetZ();
    for (current = ObjectList->Next()->Next(); current; current= current->Next())
      {
      if (current->IsBadGuy())
	{
        ObjPos[0]=current->GetX();
        ObjPos[1]=current->GetY();
        ObjPos[2]=current->GetZ();
        ObjPos[0]-=ShipX;
        ObjPos[1]-=ShipY;
        ObjPos[2]-=ShipZ;
        theta=asinf(-(ObjPos[0]/sqrt(SQR(ObjPos[0])+SQR(ObjPos[2]))));
        if (ObjPos[2]<0)
	  theta=PI-theta;
	norm=sqrt(SQR(ObjPos[0])+SQR(ObjPos[1])+SQR(ObjPos[2]));
        phi=asinf(ObjPos[1]/norm);
        current->SetOrient(-phi,-theta,0);
	norm=norm*10;
	current->AddVel(-ObjPos[0]/norm,-ObjPos[1]/norm,-ObjPos[2]/norm);
	}
      }
    }      
}

/////////////////////////////////////////////////////

int System::ResolveCollisions()
{
 int scorethisround=0;
 Object *current,*loop,*thing,*new1,*new2;
 int flag1=0,flag2=0;
 int i,sonify;;
 new1=NULL;
 new2=NULL;
// fprintf (stderr,"NEW TICK\n");
 for (current = ObjectList; current->Next();)
  {  // cycle through all objects
  for (loop = current->Next(); loop->Next();)
   {  // look for collisions with all objects after current
   if (current->Next()->CollisionDetected(loop->Next()))
    {  // A collision was detected
    if (current==ObjectList)
     {  // current is the ship (the ship was hit)
     if (loop->Next()->IsWeapon())
      {  // the ship was hit by a weapon
//      fprintf(stderr,"SHIP HIT BY WEAPON!!\n");
      if (!(new1))  // explode the weapon
       {
       new1=loop->Next()->Explode();
       }
      else InsertObject(new1,loop->Next()->Explode());
#ifdef SOUND_ON
      AUDupdate(soundhandle,"Explosion",0,NULL);
#endif SOUND_ON
      flag1=1;  // be sure to remove the weapon that exploded
      }
     else 
      {  // the ship was hit by something other than a weapon
      current->Next()->ResolveCollision(loop->Next());  // collide the objects
#ifdef SOUND_ON
      if ((!current->Next()->PreviousCollision())||(!loop->Next()->PreviousCollision()))
       AUDupdate(soundhandle,"Collision",0,NULL);
#endif SOUND_ON
      }
     EShield->LoseEnergy(); // ship was hit by something... take away 
			    // from shields
     }  // end ship hit
    else if (current->Next()->IsWeapon()||loop->Next()->IsWeapon())
     {  // object collision, one or two weapons
     sonify=0;
     if (current->Next()->IsWeapon())
       {
       if (!(loop->Next()->IsWeapon()))
         {
	 this->score+=loop->Next()->Points();
         if (loop->Next()->Points())
           sonify=1;
         }
       }
     else 
       {
       this->score+=current->Next()->Points();
       if (current->Next()->Points())
         sonify=1;
       }
       
     if (!(new1))
      {  // explode loop
      new1=loop->Next()->Explode();
      }
     else InsertObject(new1,loop->Next()->Explode());
     if (!(new2))
      {  // explode current
      new2=current->Next()->Explode();
      }
     else InsertObject(new2,current->Next()->Explode());
#ifdef SOUND_ON	       
     if (sonify)
       AUDupdate(soundhandle,"Explosion",0,NULL);
#endif SOUND_ON
     flag1=1; // remove loop later
     flag2=1; // remove current later
     }
    else
     { // two objects collided, not the ship and not a weapon
     //if ((!current->Next()->PreviousCollision())||(!loop->Next()->PreviousCollision()))
     //  AUDupdate(soundhandle,"Collision",0,NULL);
     current->Next()->ResolveCollision(loop->Next());  // collide them
     }
    }
   if (flag1)
    {  // remove loop (it exploded) and go to next current
//    fprintf (stderr,"Removing loop\n");
    if (loop->Next()->RemoveMe())
      {
      while (loop->Next()->IsLocked()) 
      {
        for (i=0;i<10000;i++);
      }
      loop->Next()->Killing();
      thing=RemoveObject(loop);
      thing->Destroy();
      }
    InsertObject(loop,new1);
    InsertObject(loop,new2);
    new1=NULL;
    new2=NULL;
    flag1=0;
//    fprintf(stderr,"Loop gone\n");
    break;
    }
   else loop=loop->Next();
   }  // end loop loop
  if (flag2)
   {  // remove current (it exploded)
   if (current->Next()->RemoveMe())
     {
     while (current->Next()->IsLocked()) 
     {
       for (i=0;i<10000;i++);
     }
     current->Next()->Killing();
     thing=RemoveObject(current);
     thing->Destroy();
     }
   flag2=0;
   }
  else current=current->Next();
  }  // end current loop
  return scorethisround;
}

/////////////////////////////////////////////////////
//Modify Position of everything relative to wand position to
//keep the fighter at the end of the wand
void System::ModifyPos()
{
  Object *current;
  float WandPos[3],shipx,shipy,shipz,WandFront[3],WandOrient[3],dx,dy,dz;
  CAVEGetPosition(CAVE_WAND,WandPos);
  CAVEGetVector(CAVE_WAND_FRONT,WandFront);
  shipx=ObjectList->Next()->GetX();     //get fighter position
  shipy=ObjectList->Next()->GetY();
  shipz=ObjectList->Next()->GetZ();
  for (current = ObjectList->Next(); current; current = current->Next())
    {
    current->MoveRel(WandPos[0]+WandFront[0]-shipx,   //move to proper position
		     WandPos[1]+WandFront[1]-shipy,
		     WandPos[2]+WandFront[2]-shipz);
    }
 EShield->Move(WandPos[0]+WandFront[0],               //move shield too
    	       WandPos[1]+WandFront[1],
     	       WandPos[2]+WandFront[2]);
 CAVEGetOrientation(CAVE_WAND,WandOrient);            //spin if necessary
 dx=WandOrient[0]*PI/180;                             //wand spin
 dy=WandOrient[1]*PI/180;
 dz=WandOrient[2]*PI/180;
 if (ObjectList->Next())                              //spin ship
   ObjectList->Next()->SetOrient(dx,dy,dz);
    
}

//////////////////////////////////////////////////////////////////////////
////Removes object from the linked list
Object *System::RemoveObject(Object *place)
{
  Object *thing;     //temporary object
  thing=NULL;
  if(place)
    {
    thing=place->Next();      //save pointer to object
    place->Link(thing->Next());    //bypass this link in the list
    }
  return(thing);              //return object to be delt with appropriately
}

/////////////////////////////////////////////////////////////////////////
//Put object in our list
void System::InsertObject(Object *place,Object *thing)
{
  Object *temp;
  if (thing)                         //if this isn't a joke
    {
    temp=thing;    
    while (temp->Next())             //while not at the end of insertable list
      {
      temp=temp->Next();             //tranverse new list to the end
      }
    temp->Link(place->Next());       //link end of new list to insert spot to insert
    place->Link(thing);              //Link old list with new list
    }
}

/////////////////////////////////////////////////////////////////////////
///Draw our lovely radar
void System::DrawRadar(Object *thing,float Rpos[],float Fpos[])
{
  glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,black);
  if (thing->IsWeapon())                               //if object is draw as yellow
    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,yellow);
  else glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,red);
  glEnable(GL_POINT_SMOOTH);
  glPointSize(3);                                     
  glBegin(GL_POINTS);
  glVertex3f(Rpos[0]+(thing->GetX()-Fpos[0])/1000.,Rpos[1]+(thing->GetY()-Fpos[1])/1000.,Rpos[2]+(thing->GetZ()-Fpos[2])/1000.);   //draw point
  glEnd();
  glDisable(GL_POINT_SMOOTH);
  glPointSize(1);
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,black);
}

////////////////////////////////////////////////////////////////////////
////make some start
void System::InitStarField()
{
  /*
  int i;
  float p[3]={0,1,0};
  float phi,theta,temp[3];

  for (i=0;i<500;i++)                 //make 500 stars
    {
    phi=(float)rand()*PI/32768;       //random placement
    theta=(float)rand()*2*PI/32768;
    ROTATE(phi,0,0,p,temp);           //put point in its place
    ROTATE(0,theta,0,temp,temp);
   
    temp[0]=75*temp[0];               //position of star from center
    temp[1]=75*temp[1];
    temp[2]=75*temp[2];
    
    stars[i][0]=temp[0];              //make star permanent
    stars[i][1]=temp[1];
    stars[i][2]=temp[2];
    
    }
    */
//  NEW STAR FIELD
      /*
  int j;
  for (i=0;i<2048;i++)
    for (j=0;j<2048;j++)
      if (((float)rand())/16384 <= 0.1)
	{
//	fprintf(stderr,"WHITE POINT     ");
	star_tex[i][j][0]=255;
	star_tex[i][j][1]=255;
	star_tex[i][j][2]=255;
	star_tex[i][j][3]=255;
	}
      else 
	{
	star_tex[i][j][0]=0;
	star_tex[i][j][1]=0;
	star_tex[i][j][2]=0;
	star_tex[i][j][3]=255;
	}       
	*/
}

///////////////////////////////////////////////////////////////////////
//Draw our star field////////////////////////
void System::DrawStarField()
{
  
//  int i;

 float x[3]={1,0,0};
 float y[3]={0,1,0};
 float z[3]={0,0,1};

  glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,black);
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,white);
  /*
  glBegin(GL_POINTS);
  //glPointSize(1);       //one pixel
  for (i=0;i<500;i++)    //draw all 500 stars
    {
    glVertex3f(stars[i][0],stars[i][1],stars[i][2]);        //put star in its place
    }
  glEnd();
  */
  glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,black);  

  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
  glBindTextureEXT (GL_TEXTURE_2D, texName);
  glEnable(GL_TEXTURE_2D);
  glPushMatrix();
  
//  glRotatef(star_rot[2],0,0,1);
  
//  glPushMatrix();

  glRotatef(star_rot[0],0,1,0);

  ROT_AXIS(y,-star_rot[0]*PI/(float)180,x,x);
  
//  glPushMatrix();
  
  glRotatef(star_rot[1],x[0],x[1],x[2]);

//  if ((CAVEWall==CAVE_FRONT_WALL)&&(CAVEEye==CAVE_RIGHT_EYE))
//    fprintf(stderr,"%f, %f\n",star_rot[0],star_rot[1]);

//  gluQuadricOrientation(sphereObj,GLU_INSIDE);
  gluQuadricTexture(sphereObj,GL_TRUE);
  gluSphere(sphereObj,100,16,16);
  glDisable(GL_TEXTURE_2D);
//  gluQuadricOrientation(sphereObj,GLU_OUTSIDE);
  gluQuadricTexture(sphereObj,GL_FALSE);
//  glPopMatrix();
//  glPopMatrix();
  glPopMatrix();


}

////////////////////////////////////////////////////////////
//your score is////////////////////////////
void System::DrawScore()
{
  glPushMatrix();
  glRotatef(90,1,0,0);
  glTranslatef(-5, -5, -5);
  glScalef(.01,.01,.01);
  glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,red);
  score_text();
  glTranslatef(0,0,25);
  score_nums(this->score);
  glPopMatrix();
}


    
///////////////////////////////////////////////////////////////////////
//stop that darn racket/////////
void System::KillSound()
{
#ifdef SOUND_ON
  AUDterminate(soundhandle);
  EndSoundServer();
#endif SOUND_ON
}

#endif SYSTEM_H
